@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2011 Christian Grothoff (and other contributing authors)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -624,7 +624,7 @@ create_response (void *cls,
 					      (void *) METHOD_ERROR,
 					      MHD_RESPMEM_PERSISTENT);
   ret = MHD_queue_response (connection,
-			    MHD_HTTP_METHOD_NOT_ACCEPTABLE,
+			    MHD_HTTP_NOT_ACCEPTABLE,
 			    response);
   MHD_destroy_response (response);
   return ret;
@@ -747,4 +747,3 @@ main (int argc, char *const *argv)
   MHD_stop_daemon (d);
   return 0;
 }
-
